@@ -14,7 +14,11 @@ DRBL_SCRIPT_PATH="${DRBL_SCRIPT_PATH:-/opt/drbl/}"
 . $DRBL_SCRIPT_PATH/sbin/drbl-conf-functions
 
 # Settings
-SL_ISO_URL_EXAMPLE="http://prdownloads.sourceforge.net/clonezilla/clonezilla-live-1.2.5-5-i686.iso"
+SL_ISO_URL_EXAMPLE="http://prdownloads.sourceforge.net/clonezilla/clonezilla-live-1.2.5-6-i686.iso"
+# The method for netboot's root file system. "fetch" means the client will download the filesystem.squashfs from tftpd server, "nfsroot" means client will mount the nfs server. By default we use fetch. "nfsroot" only works for Debian-live based system (e.g. Clonezilla live, GParted live...)
+rootfs_location="fetch"
+# Force to update the PXELinux config file
+force_to_update_pxecfg="yes"
 
 # List all supported SL_NAME
 # // On 2009/05/25, we remove the support for PLD, INSERT, PUD, since those projects are not developed any more. As for GeeXbox, since we do not have time, it's removed, too.
@@ -42,6 +46,8 @@ USAGE() {
   echo "-b, --batch-mode   Run clone in batch mode."
   echo "-i, --install ISO:        Load Small Linux ISO into DRBL environment, you must put the iso file in the current working dir." 
   echo "-d, --distribution DIST:  Assign the small GNU/Linux distribution as DIST. Available names are: $supported_SL_dists."
+  echo "-f, --use-nfsroot      By default the boot parameter to load the root file system (e.g. filesystem.squashfs) is fetch. With this option, nfsroot is used. //NOTE// This function only works for live-initramfs based system. E.g. Clonezilla live or GParted live."
+  echo "-k, --keep-pxecfg  Keep the pxelinux config file without updating the boot parameters."
   echo "-n, --no-prompt-boot-message  Skip the prompt messages about booting via PXE or etherboot in the end of execution."
   echo "-u, --uninstall DIST:     Uninstall Small Linux DIST."
   echo "-V, --dist-version VER:   Assign the version of small GNU/Linux version number as VER."
@@ -57,6 +63,15 @@ USAGE() {
   done
   echo "    To remove all Small Linux, run '$0 -u all'"
 }
+#
+get_rootfs_location_opt() {
+rootfs_location_opt=""
+case "$rootfs_location" in
+  "fetch") rootfs_location_opt="fetch=tftp://$server_IP/$squashfs_file" ;;
+  "nfsroot")
+           rootfs_location_opt="netboot=nfs nfsroot=$server_IP:$drbl_common_root/clonezilla-live/" ;;
+esac
+} # end of get_rootfs_location_opt
 # get rootfile
 get_SL_rootfile(){
   # this function must be used after iso is mounted, we need to find version inside it for some distribution (such as PLD).
@@ -93,21 +108,23 @@ prepare_param_in_pxe_cfg(){
     clonezilla-live)
        # Find the IP address of this server. Here we choose the first one in the list, since it's just an example.
        server_IP="$(LC_ALL=C get-all-nic-ip -i | awk -F" " '{print $1}')"
+       get_rootfs_location_opt
        if [ -z "$cl_gp_boot_param" ]; then
 	 # We do not put ip=frommedia because the /etc/resolv.conf got in live-initramfs won't exist after initramfs is done.        
-         append_param_in_pxe_cfg="initrd=$nbi_initrd boot=live union=aufs noswap noprompt nolocales vga=788 fetch=tftp://$server_IP/$squashfs_file ocs_server=\"$server_IP\""
+         append_param_in_pxe_cfg="initrd=$nbi_initrd boot=live union=aufs noswap noprompt nolocales vga=788 $rootfs_location_opt ocs_server=\"$server_IP\""
        else
-         append_param_in_pxe_cfg="initrd=$nbi_initrd $cl_gp_boot_param noprompt fetch=tftp://$server_IP/$squashfs_file ocs_server=\"$server_IP\""
+         append_param_in_pxe_cfg="initrd=$nbi_initrd $cl_gp_boot_param noprompt $rootfs_location_opt ocs_server=\"$server_IP\""
        fi
        ;;
     gparted-live)
        # Find the IP address of this server. Here we choose the first one in the list, since it's just an example.
        server_IP="$(LC_ALL=C get-all-nic-ip -i | awk -F" " '{print $1}')"
+       get_rootfs_location_opt
        if [ -z "$cl_gp_boot_param" ]; then
 	 # We do not put ip=frommedia because the /etc/resolv.conf got in live-initramfs won't exist after initramfs is done.        
-         append_param_in_pxe_cfg="initrd=$nbi_initrd boot=live union=aufs noswap noprompt nolocales vga=788 fetch=tftp://$server_IP/$squashfs_file "
+         append_param_in_pxe_cfg="initrd=$nbi_initrd boot=live union=aufs noswap noprompt nolocales vga=788 $rootfs_location_opt "
        else
-         append_param_in_pxe_cfg="initrd=$nbi_initrd $cl_gp_boot_param noprompt fetch=tftp://$server_IP/$squashfs_file"
+         append_param_in_pxe_cfg="initrd=$nbi_initrd $cl_gp_boot_param noprompt $rootfs_location_opt"
        fi
        ;;
   esac
@@ -342,8 +359,15 @@ put_kernel_initrd_root_fs_on_pxe_server() {
   cp -af $kernel $pxecfg_pd/$nbi_kernel
   cp -af $oldinitrd $pxecfg_pd/$nbi_initrd
   get_SL_rootfile
-  echo "Copying root image... $msg_this_might_take_several_minutes"
-  cp -fv $isomnt/$rootfile $pxecfg_pd/$squashfs_file
+  case "$rootfs_location" in
+    "fetch") 
+             echo "Copying root image $isomnt/$rootfile to $pxecfg_pd... $msg_this_might_take_several_minutes"
+	     rsync -aP $isomnt/$rootfile $pxecfg_pd/$squashfs_file ;;
+    "nfsroot")
+             echo "Copying root image $isomnt/$rootfile to $drbl_common_root/clonezilla-live... $msg_this_might_take_several_minutes"
+	     mkdir -p $drbl_common_root/clonezilla-live/live/
+             rsync -aP $isomnt/$rootfile $drbl_common_root/clonezilla-live/live/ ;;
+  esac
   prepare_param_in_pxe_cfg
   chmod 644 $pxecfg_pd/$nbi_kernel $pxecfg_pd/$nbi_initrd
 } # end of put_kernel_initrd_root_fs_on_pxe_server
@@ -399,7 +423,11 @@ install_SL(){
     exit 1
   fi
   
-  if [ -z "$(file $SL_ISO | grep -i "ISO 9660 CD-ROM filesystem data")" ]; then
+  # Program file might give different results:
+  # ISO 9660 CD-ROM filesystem data # file vesion 5.03
+  # x86 boot sector # file version 4.10
+  if [ -z "$(LC_ALL=C file -L $SL_ISO | grep -i "ISO 9660 CD-ROM filesystem data")" ] && \
+     [ -z "$(LC_ALL=C file -L $SL_ISO | grep -i "x86 boot sector")" -a "$(echo $SL_ISO | grep -iE "\.iso")" ]; then
     [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
     echo "$SL_ISO is not an ISO 9660 CD-ROM filesystem data file!"
     [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
@@ -524,8 +552,8 @@ install_SL(){
     # In RH9: vmlinuz-2.4.20-28.9smp: x86 boot sector
     if [ -n "$(file $i | grep -iE "(Linux x86 kernel|Linux kernel x86 boot executable|x86 boot sector)")" ]; then
       # Ugly
-      # we skip memtest
-      if [ -z "$(echo $i | grep -iE "(memtest|eb.zli|memdisk|gpxe.lkn)")" ]; then
+      # we skip memtest, etherboot, freedos...
+      if [ -z "$(echo $i | grep -iE "(memtest|eb.zli|memdisk|gpxe.lkn|freedos|fdos)")" ]; then
         kernel="$i"
         break
       fi
@@ -625,7 +653,11 @@ install_SL(){
   umount_and_clean_tmp_working_dirs
   
   # append the config file in pxelinux dir.
-  if [ -z "$(grep -E "^[[:space:]]*label[[:space:]]+$SL_NAME[[:space:]]*$" $PXELINUX_DIR/default)" ]; then
+  if [ -z "$(grep -E "^[[:space:]]*label[[:space:]]+$SL_NAME[[:space:]]*$" $PXELINUX_DIR/default)" ] || [ "$force_to_update_pxecfg" = "yes" ]; then
+    if [ -n "$(grep -E "^[[:space:]]*label[[:space:]]+$SL_NAME[[:space:]]*$" $PXELINUX_DIR/default)" ]; then
+      echo "First removing $SL_NAME setting in $PXELINUX_DIR/default..."
+      delete_label_block_pxe_img $SL_NAME $PXELINUX_DIR/default
+    fi
     echo "Append the $SL_FULL_NAME config in $PXELINUX_DIR/default..."
     cat <<-SL_PXE >> $PXELINUX_DIR/default 
 label $SL_NAME
@@ -681,6 +713,9 @@ uninstall_SL(){
       echo "Removing installed $id if it exists..."
       rm -fv $pxecfg_pd/${id}-*
     fi
+    if [ -d "$drbl_common_root/clonezilla-live/" ]; then
+      rm -rfv $drbl_common_root/clonezilla-live/*
+    fi
   done
 } # end of uninstall_SL
 
@@ -721,6 +756,8 @@ while [ $# -gt 0 ]; do
 	  shift
         fi
 	;;
+    -f|--use-nfsroot) shift; rootfs_location="nfsroot" ;;
+    -k|--keep-pxecfg) shift; force_to_update_pxecfg="no" ;;
     -n|--no-prompt-boot-message) shift; booting_messages="off";;
     -u|--uninstall)
         shift; mode="uninstall"
